@@ -20,10 +20,6 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
 #include <string.h>
 #include <errno.h>
 #include "gtksourcebufferoutputstream.h"
@@ -636,7 +632,7 @@ validate_and_insert (GtkSourceBufferOutputStream *stream,
 
 			ptr = g_utf8_find_prev_char (buffer, buffer + len);
 
-			if (ptr && *ptr == '\r' && ptr - buffer == (glong)len - 1)
+			if (ptr && *ptr == '\r' && ptr - buffer == len - 1)
 			{
 				stream->priv->buffer = g_new (gchar, 2);
 				stream->priv->buffer[0] = '\r';
@@ -657,7 +653,7 @@ validate_and_insert (GtkSourceBufferOutputStream *stream,
 
 			apply_error_tag (stream);
 
-			if ((nvalid != len || !owned) && buffer[nvalid] != '\0')
+			if (nvalid != len && buffer[nvalid] != '\0')
 			{
 				/* make sure the buffer is always properly null
 				 * terminated. This is needed, at least for now,
@@ -768,7 +764,6 @@ end_append_text_to_document (GtkSourceBufferOutputStream *stream)
 	gtk_text_buffer_set_modified (GTK_TEXT_BUFFER (stream->priv->source_buffer),
 	                              FALSE);
 
-	gtk_text_buffer_end_user_action (GTK_TEXT_BUFFER (stream->priv->source_buffer));
 	gtk_source_buffer_end_not_undoable_action (stream->priv->source_buffer);
 }
 
@@ -946,13 +941,8 @@ gtk_source_buffer_output_stream_write (GOutputStream  *stream,
 			g_free (from_charset);
 		}
 
-		/* Begin not undoable action. Begin also a normal user action,
-		 * since we load the file chunk by chunk and it should be seen
-		 * as only one action, for the features that rely on the user
-		 * action.
-		 */
+		/* Init the undoable action */
 		gtk_source_buffer_begin_not_undoable_action (ostream->priv->source_buffer);
-		gtk_text_buffer_begin_user_action (GTK_TEXT_BUFFER (ostream->priv->source_buffer));
 
 		gtk_text_buffer_get_start_iter (GTK_TEXT_BUFFER (ostream->priv->source_buffer),
 		                                &ostream->priv->pos);
