@@ -487,6 +487,53 @@ gtk_source_encoding_get_name (const GtkSourceEncoding* enc)
 	return (enc->name == NULL) ? _("Unknown") : _(enc->name);
 }
 
+/**
+ * gtk_source_encoding_get_default_candidates:
+ *
+ * Gets the list of default candidate encodings to try when loading a file. See
+ * gtk_source_file_loader_set_candidate_encodings().
+ *
+ * This function returns a different list depending on the current locale (i.e.
+ * language, country and default encoding).
+ *
+ * Returns: (transfer container) (element-type GtkSource.Encoding): the list of
+ * default candidate encodings. Free with g_slist_free().
+ * Since: 3.14
+ */
+GSList *
+gtk_source_encoding_get_default_candidates (void)
+{
+	const gchar *encodings_str;
+	GVariant *encodings_variant;
+	const gchar **encodings_strv;
+	GSList *encodings_list;
+
+	/* Translators: This is the sorted list of encodings used by
+	 * GtkSourceView for automatic detection of the file encoding. You may
+	 * want to customize it adding encodings that are common in your
+	 * country, for instance the GB18030 encoding for the Chinese
+	 * translation. You may also want to remove the ISO-8859-15 encoding
+	 * (covering English and most Western European languages) if you think
+	 * people in your country will rarely use it.  "CURRENT" is a magic
+	 * value used by GtkSourceView and it represents the encoding for the
+	 * current locale, so please don't translate the "CURRENT" term.  Only
+	 * recognized encodings are used. See
+	 * https://git.gnome.org/browse/gtksourceview/tree/gtksourceview/gtksourceencoding.c#n147
+	 * for a list of supported encodings.
+	 */
+	encodings_str = _("['UTF-8', 'CURRENT', 'ISO-8859-15', 'UTF-16']");
+
+	encodings_variant = g_variant_new_parsed (encodings_str);
+	g_variant_ref_sink (encodings_variant);
+
+	encodings_strv = g_variant_get_strv (encodings_variant, NULL);
+
+	encodings_list = _gtk_source_encoding_strv_to_list (encodings_strv);
+
+	g_variant_unref (encodings_variant);
+	return encodings_list;
+}
+
 /**
  * gtk_source_encoding_copy:
  * @enc: a #GtkSourceEncoding.
@@ -518,7 +565,6 @@ gtk_source_encoding_free (GtkSourceEncoding *enc)
 	g_return_if_fail (enc != NULL);
 }
 
-#if 0
 /* Will probably be used in the future. */
 static gboolean
 data_exists (GSList         *list,
@@ -564,6 +610,7 @@ _gtk_source_encoding_strv_to_list (const gchar * const *enc_str)
 	return g_slist_reverse (res);
 }
 
+#if 0
 gchar **
 _gtk_source_encoding_list_to_strv (const GSList *enc_list)
 {
