@@ -60,15 +60,18 @@ const GtkSourceEncoding	*gtk_source_encoding_get_current	(void);
 void			 gtk_source_encoding_foreach		(GtkSourceEncodingForeachFunc func,
 								 gpointer                     user_data);
 
+GSList			*gtk_source_encoding_get_default_candidates
+								(void);
+
 /* These should not be used, they are just to make python bindings happy */
 GtkSourceEncoding	*gtk_source_encoding_copy		(const GtkSourceEncoding *enc);
 void			 gtk_source_encoding_free		(GtkSourceEncoding       *enc);
 
-#if 0
-/* Will probably be used in the future. */
 G_GNUC_INTERNAL
 GSList			*_gtk_source_encoding_strv_to_list	(const gchar * const     *enc_str);
 
+#if 0
+/* Will probably be used in the future. */
 G_GNUC_INTERNAL
 gchar		       **_gtk_source_encoding_list_to_strv	(const GSList            *enc);
 #endif
